/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include <gtk/gtk.h>
#include <artnet/artnet.h>
#include <stdlib.h>
#include <string.h>
#include "memory.h"
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>


// storage for scenes
uint8_t scene_data[SCENES][ARTNET_DMX_LENGTH] ;

// storage for flashes
uint8_t flash_data[FLASHES][ARTNET_DMX_LENGTH] ;

//storage for clipboard
uint8_t clipboard[SLIDERS] ;

// storage for slow mode
uint8_t sm_data[MAX_SLOW_SCENES][SLIDERS] ;

beat_scene beat_data[MAX_BEAT_SCENES];


int slow_scenes = 0;

int beat_scenes = 0;
/*
 *
 *
 *
 */
 
 
/*
 * Clear all the programmable memories
 *
 */
void init_memory() {
	memset(scene_data, 0x00, sizeof(scene_data) ) ;
	memset(flash_data, 0x00, sizeof(flash_data) ) ;
	memset(clipboard, 0x00, sizeof(clipboard) ) ;
	memset(sm_data, 0x00, sizeof(sm_data) ) ;
}






/*
 * Copies the current state of the sliders to the clipboard
 *
 */
int copy_to_clipboard(uint8_t *data, int length) {
	memcpy(clipboard, data, MIN(length, ARTNET_DMX_LENGTH) ) ;
	return MIN(length, ARTNET_DMX_LENGTH) ;
}

/*
 * Gets length number of channels from the clipboard
 * 
 * Returns the number of bytes copied
 */
uint8_t *get_from_clipboard(int *length) {
	*length = sizeof(clipboard) ;
	return clipboard ;
}


/*
 * Copies the current state of the sliders to the slow mode memory
 *
 */
int copy_to_slow(uint8_t *data, int length) {
	if(slow_scenes == MAX_SLOW_SCENES) {
		return -1 ;
	}
	
	printf("adding to slow in pos %d\n", slow_scenes) ;
	memcpy(sm_data[slow_scenes], data, MIN(length, SLIDERS) ) ;
	slow_scenes++ ;
	return MIN(length, SLIDERS) ;
}

int get_slow_scene_count() {
	return slow_scenes ;	
}

/*
 * Gets length number of channels from the slow mode memory
 * 
 * Returns the number of bytes copied
 */
uint8_t *get_from_slow(int id, int *length) {
	if (id < 0 || id >= MAX_SLOW_SCENES) {
		return NULL ;
	}
	printf("getting slow scene %d\n", id) ;
	
	*length = SLIDERS ;
	return sm_data[id] ;
}




/*
 * Store data of length in the flash with id
 *
 */
int store_flash(int id, uint8_t *data, int length) {
	if (id < 0 || id >= FLASHES) {
		return -1 ;
	}
	
	memcpy(flash_data[id], data, MIN(length, ARTNET_DMX_LENGTH)) ;
	
	return 0 ;
}


/*
 * Store data of length in the scene with id
 *
 */
int store_scene(int id, uint8_t *data, int length) {
	if (id < 0 || id >= SCENES) {
		return -1 ;
	}
	
	memcpy(scene_data[id], data, MIN(length, ARTNET_DMX_LENGTH)) ;
	
	return 0 ;
}



uint8_t *get_flash(int id, int *length) {
	*length = SLIDERS ;
	return flash_data[id] ;
}

uint8_t *get_scene(int id, int *length) {
	return NULL ;
}

int load_from_file(const char *filename) {
	int ret ;
	FILE *file = fopen(filename, "rb") ;
	
	
	if(file == NULL) {
		printf("open failed\n") ;
		return -1 ;
	}
	
	ret = fread(flash_data, 1, sizeof(flash_data) , file) ;
	
	if(ret != sizeof(flash_data) ) {
		printf("save 1 failed %d %d\n", ret, sizeof(flash_data) ) ;
		return -1;
	}

	ret = fread(scene_data, 1, sizeof(scene_data) , file) ;
	
	if(ret != sizeof(scene_data) ) {
		printf("save 1 failed %d %d\n", ret, sizeof(scene_data) ) ;
		return -1;
	}
	
	// read number of slow scenes
	ret = fread(&slow_scenes, 1, sizeof(slow_scenes) , file) ;
	
	if(ret != sizeof(slow_scenes) ) {
		printf("save 2 failed %d %d\n", ret, sizeof(slow_scenes) ) ;
		return -1;
	}
	
	// read slow scene data
	ret = fread(sm_data, 1, slow_scenes * sizeof(sm_data) / MAX_SLOW_SCENES , file) ;
	
	if(ret != slow_scenes * sizeof(sm_data) / MAX_SLOW_SCENES ) {
		printf("save 1 failed %d %d\n", ret, slow_scenes * sizeof(sm_data) / MAX_SLOW_SCENES ) ;
		return -1;
	}
	
	
	// write number of beat scenes
	ret = fread(&beat_scenes, 1, sizeof(beat_scenes) , file) ;
	
	if(ret != sizeof(beat_scenes) ) {
		printf("save 4 failed %d %d\n", ret, sizeof(beat_scenes) ) ;
		return -1;
	}
	
	// write slow scene data
	ret = fread(beat_data, 1, beat_scenes * sizeof(beat_data) / MAX_BEAT_SCENES , file) ;
	
	if(ret != beat_scenes * sizeof(beat_data) / MAX_BEAT_SCENES  ) {
		printf("save 5 failed %d %d\n", ret, beat_scenes * sizeof(sm_data) / MAX_BEAT_SCENES  ) ;
		return -1;
	}
	
	fclose(file) ;	
	
	return 0;
}


/*
 * These are quick and dirty hacks, shouw set up a proper format for show files
 *
 */
int save_to_file(const char *filename) {
	int ret ;
	FILE *file = fopen(filename, "wb") ;
	
	
	if(file == NULL) {
		printf("open failed\n") ;
		return -1 ;
	}
	
	// write flash data
	ret = fwrite(flash_data, 1, sizeof(flash_data) , file) ;
	
	if(ret != sizeof(flash_data) ) {
		printf("save 1 failed %d %d\n", ret, sizeof(flash_data) ) ;
		return -1;
	}

	// write scene data
	ret = fwrite(scene_data, 1, sizeof(scene_data) , file) ;
	
	if(ret != sizeof(scene_data) ) {
		printf("save 1 failed %d %d\n", ret, sizeof(scene_data) ) ;
		return -1;
	}
	
	// write number of slow scenes
	ret = fwrite(&slow_scenes, 1, sizeof(slow_scenes) , file) ;
	
	if(ret != sizeof(slow_scenes) ) {
		printf("save 2 failed %d %d\n", ret, sizeof(slow_scenes) ) ;
		return -1;
	}
	
	// write slow scene data
	ret = fwrite(sm_data, 1, slow_scenes * sizeof(sm_data) / MAX_SLOW_SCENES , file) ;
	
	if(ret != slow_scenes * sizeof(sm_data) / MAX_SLOW_SCENES  ) {
		printf("save 3 failed %d %d\n", ret, slow_scenes * sizeof(sm_data) / MAX_SLOW_SCENES  ) ;
		return -1;
	}
	
	// write number of beat scenes
	ret = fwrite(&beat_scenes, 1, sizeof(beat_scenes) , file) ;
	
	if(ret != sizeof(beat_scenes) ) {
		printf("save 4 failed %d %d\n", ret, sizeof(beat_scenes) ) ;
		return -1;
	}
	
	// write slow scene data
	ret = fwrite(beat_data, 1, beat_scenes * sizeof(beat_data) / MAX_BEAT_SCENES , file) ;
	
	if(ret != beat_scenes * sizeof(beat_data) / MAX_BEAT_SCENES  ) {
		printf("save 5 failed %d %d\n", ret, beat_scenes * sizeof(sm_data) / MAX_BEAT_SCENES  ) ;
		return -1;
	}
	
	fclose(file) ;	
	
	return 0;
}







/*
 * Copies the current state of the sliders to the slow mode memory
 *
 */
int copy_to_beat(beat_scene *scene) {
	if(beat_scenes == MAX_BEAT_SCENES) {
		return -1 ;
	}

	memcpy(&beat_data[beat_scenes], scene, sizeof(beat_scene) ) ;
	beat_scenes++ ;
	return 1 ;
}

int get_beat_scene_count() {
	return beat_scenes ;	
}

/*
 * Gets length number of channels from the slow mode memory
 * 
 * Returns the number of bytes copied
 */
int get_from_beat(int id, beat_scene *tmp ) {

	if (id < 0 || id >= MAX_BEAT_SCENES) {
		return -1 ;
	}
	printf("getting beat scene %d\n", id) ;
	
	printf("%d is %d\n", 0, beat_data[0].defn[1].param1) ;
	
	memcpy(tmp, &beat_data[id], sizeof(beat_scene) ) ;
	return 0;
}
